@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpctl.c,v 1.110 2006/08/28 05:28:49 henning Exp $ */
+/*	$OpenBSD: bgpctl.c,v 1.126 2007/08/06 19:16:06 sobrado Exp $ */
 
 /*
  * Copyright (c) 2003 Henning Brauer <henning@openbsd.org>
@@ -36,6 +36,7 @@
 #include "rde.h"
 #include "log.h"
 #include "parser.h"
+#include "irrfilter.h"
 
 enum neighbor_views {
 	NV_DEFAULT,
@@ -48,6 +49,7 @@ char		*fmt_peer(const char *, const struct bgpd_addr *, int, int);
 void		 show_summary_head(void);
 int		 show_summary_msg(struct imsg *, int);
 int		 show_summary_terse_msg(struct imsg *, int);
+int		 show_neighbor_terse(struct imsg *);
 int		 show_neighbor_msg(struct imsg *, enum neighbor_views);
 void		 print_neighbor_capa_mp_safi(u_int8_t);
 void		 print_neighbor_msgstats(struct peer *);
@@ -87,8 +89,8 @@ usage(void)
 {
 	extern char	*__progname;
 
-	fprintf(stderr, "usage: %s [-s socket] [-n] "
-	    "<command> [arg [...]]\n", __progname);
+	fprintf(stderr, "usage: %s [-n] [-o directory] [-s socket] "
+	    "command [arguments ...]\n", __progname);
 	exit(1);
 }
 
@@ -102,16 +104,20 @@ main(int argc, char *argv[])
 	struct parse_result	*res;
 	struct ctl_neighbor	 neighbor;
 	struct ctl_show_rib_request	ribreq;
-	char			*sockname;
+	char			*sockname, *outdir;
 	enum imsg_type		 type;
 
 	sockname = SOCKET_NAME;
-	while ((ch = getopt(argc, argv, "ns:")) != -1) {
+	outdir = getcwd(NULL, 0);
+	while ((ch = getopt(argc, argv, "no:s:")) != -1) {
 		switch (ch) {
 		case 'n':
 			if (++nodescr > 1)
 				usage();
 			break;
+		case 'o':
+			outdir = optarg;
+			break;
 		case 's':
 			sockname = optarg;
 			break;
@@ -126,6 +132,9 @@ main(int argc, char *argv[])
 	if ((res = parse(argc, argv)) == NULL)
 		exit(1);
 
+	if (res->action == IRRFILTER)
+		irr_main(res->as.as, res->flags, outdir);
+
 	memcpy(&neighbor.addr, &res->peeraddr, sizeof(neighbor.addr));
 	strlcpy(neighbor.descr, res->peerdesc, sizeof(neighbor.descr));
 
@@ -147,6 +156,7 @@ main(int argc, char *argv[])
 
 	switch (res->action) {
 	case NONE:
+	case IRRFILTER:
 		usage();
 		/* not reached */
 	case SHOW:
@@ -185,6 +195,7 @@ main(int argc, char *argv[])
 		break;
 	case SHOW_NEIGHBOR:
 	case SHOW_NEIGHBOR_TIMERS:
+	case SHOW_NEIGHBOR_TERSE:
 		if (res->peeraddr.af || res->peerdesc[0])
 			imsg_compose(ibuf, IMSG_CTL_SHOW_NEIGHBOR, 0, 0, -1,
 			    &neighbor, sizeof(neighbor));
@@ -204,6 +215,12 @@ main(int argc, char *argv[])
 			ribreq.prefixlen = res->prefixlen;
 			type = IMSG_CTL_SHOW_RIB_PREFIX;
 		}
+		if (res->community.as != COMMUNITY_UNSET &&
+		    res->community.type != COMMUNITY_UNSET) {
+			memcpy(&ribreq.community, &res->community,
+			    sizeof(res->community));
+			type = IMSG_CTL_SHOW_RIB_COMMUNITY;
+		}
 		memcpy(&ribreq.neighbor, &neighbor,
 		    sizeof(ribreq.neighbor));
 		ribreq.af = res->af;
@@ -326,6 +343,9 @@ main(int argc, char *argv[])
 			case SHOW_NEIGHBOR_TIMERS:
 				done = show_neighbor_msg(&imsg, NV_TIMERS);
 				break;
+			case SHOW_NEIGHBOR_TERSE:
+				done = show_neighbor_terse(&imsg);
+				break;
 			case SHOW_RIB:
 				if (res->flags & F_CTL_DETAIL)
 					done = show_rib_detail_msg(&imsg,
@@ -352,6 +372,7 @@ main(int argc, char *argv[])
 			case NETWORK_ADD:
 			case NETWORK_REMOVE:
 			case NETWORK_FLUSH:
+			case IRRFILTER:
 				break;
 			}
 			imsg_free(&imsg);
@@ -392,8 +413,8 @@ fmt_peer(const char *descr, const struct bgpd_addr *remote_addr,
 void
 show_summary_head(void)
 {
-	printf("%-20s %-5s %-10s %-10s %-5s %-8s %s\n", "Neighbor", "AS",
-	    "MsgRcvd", "MsgSent", "OutQ", "Up/Down", "State/PrefixRcvd");
+	printf("%-20s %-8s %-10s %-10s %-5s %-8s %s\n", "Neighbor", "AS",
+	    "MsgRcvd", "MsgSent", "OutQ", "Up/Down", "State/PrfRcvd");
 }
 
 int
@@ -409,8 +430,8 @@ show_summary_msg(struct imsg *imsg, int nodescr)
 		    p->conf.remote_masklen, nodescr);
 		if (strlen(s) >= 20)
 			s[20] = 0;
-		printf("%-20s %5u %10llu %10llu %5u %-8s ",
-		    s, p->conf.remote_as,
+		printf("%-20s %8s %10llu %10llu %5u %-8s ",
+		    s, log_as(p->conf.remote_as),
 		    p->stats.msg_rcvd_open + p->stats.msg_rcvd_notification +
 		    p->stats.msg_rcvd_update + p->stats.msg_rcvd_keepalive +
 		    p->stats.msg_rcvd_rrefresh,
@@ -423,7 +444,9 @@ show_summary_msg(struct imsg *imsg, int nodescr)
 			printf("%6u", p->stats.prefix_cnt);
 			if (p->conf.max_prefix != 0)
 				printf("/%u", p->conf.max_prefix);
-		} else
+		} else if (p->conf.template)
+			printf("Template");
+		else
 			printf("%s", statenames[p->state]);
 		printf("\n");
 		free(s);
@@ -448,8 +471,8 @@ show_summary_terse_msg(struct imsg *imsg, int nodescr)
 		p = imsg->data;
 		s = fmt_peer(p->conf.descr, &p->conf.remote_addr,
 		    p->conf.remote_masklen, nodescr);
-		printf("%s %u %s\n", s, p->conf.remote_as,
-		    statenames[p->state]);
+		printf("%s %s %s\n", s, log_as(p->conf.remote_as),
+		    p->conf.template ? "Template" : statenames[p->state]);
 		free(s);
 		break;
 	case IMSG_CTL_END:
@@ -461,6 +484,36 @@ show_summary_terse_msg(struct imsg *imsg, int nodescr)
 	return (0);
 }
 
+int
+show_neighbor_terse(struct imsg *imsg)
+{
+	struct peer		*p;
+
+	switch (imsg->hdr.type) {
+	case IMSG_CTL_SHOW_NEIGHBOR:
+		p = imsg->data;
+		printf("%llu %llu %llu %llu %llu %llu %llu "
+		    "%llu %llu %llu %u %u %llu %llu %llu %llu\n",
+		    p->stats.msg_sent_open, p->stats.msg_rcvd_open,
+		    p->stats.msg_sent_notification,
+		    p->stats.msg_rcvd_notification,
+		    p->stats.msg_sent_update, p->stats.msg_rcvd_update,
+		    p->stats.msg_sent_keepalive, p->stats.msg_rcvd_keepalive,
+		    p->stats.msg_sent_rrefresh, p->stats.msg_rcvd_rrefresh,
+		    p->stats.prefix_cnt, p->conf.max_prefix,
+		    p->stats.prefix_sent_update, p->stats.prefix_rcvd_update,
+		    p->stats.prefix_sent_withdraw,
+		    p->stats.prefix_rcvd_withdraw);
+		break;
+	case IMSG_CTL_END:
+		return (1);
+	default:
+		break;
+	}
+
+	return (0);
+}
+
 int
 show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 {
@@ -480,7 +533,8 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 			    p->conf.remote_masklen) == -1)
 				err(1, NULL);
 		} else
-			if ((s = strdup(log_addr(&p->conf.remote_addr))) == NULL)
+			if ((s = strdup(log_addr(&p->conf.remote_addr))) ==
+			    NULL)
 				err(1, "strdup");
 
 		ina.s_addr = p->remote_bgpid;
@@ -489,7 +543,7 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 		if (p->conf.remote_as == 0 && p->conf.template)
 			printf("remote AS: accept any");
 		else
-			printf("remote AS %u", p->conf.remote_as);
+			printf("remote AS %s", log_as(p->conf.remote_as));
 		if (p->conf.template)
 			printf(", Template");
 		if (p->conf.cloned)
@@ -509,7 +563,8 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 		    fmt_timeframe(p->stats.last_read),
 		    p->holdtime, p->holdtime/3);
 		if (p->capa.peer.mp_v4 || p->capa.peer.mp_v6 ||
-		    p->capa.peer.refresh) {
+		    p->capa.peer.refresh || p->capa.peer.restart ||
+		    p->capa.peer.as4byte) {
 			printf("  Neighbor capabilities:\n");
 			if (p->capa.peer.mp_v4) {
 				printf("    Multiprotocol extensions: IPv4");
@@ -523,6 +578,8 @@ show_neighbor_msg(struct imsg *imsg, enum neighbor_views nv)
 				printf("    Route Refresh\n");
 			if (p->capa.peer.restart)
 				printf("    Graceful Restart\n");
+			if (p->capa.peer.as4byte)
+				printf("    4-byte AS numbers\n");
 		}
 		printf("\n");
 		switch (nv) {
@@ -584,9 +641,6 @@ print_neighbor_capa_mp_safi(u_int8_t safi)
 	case SAFI_MULTICAST:
 		printf(" Multicast");
 		break;
-	case SAFI_BOTH:
-		printf(" Unicast and Multicast");
-		break;
 	default:
 		printf(" unknown (%u)", safi);
 		break;
@@ -609,13 +663,19 @@ print_neighbor_msgstats(struct peer *p)
 	    p->stats.msg_sent_keepalive, p->stats.msg_rcvd_keepalive);
 	printf("  %-15s %10llu %10llu\n", "Route Refresh",
 	    p->stats.msg_sent_rrefresh, p->stats.msg_rcvd_rrefresh);
-	printf("  %-15s %10llu %10llu\n", "Total",
+	printf("  %-15s %10llu %10llu\n\n", "Total",
 	    p->stats.msg_sent_open + p->stats.msg_sent_notification +
 	    p->stats.msg_sent_update + p->stats.msg_sent_keepalive +
 	    p->stats.msg_sent_rrefresh,
 	    p->stats.msg_rcvd_open + p->stats.msg_rcvd_notification +
 	    p->stats.msg_rcvd_update + p->stats.msg_rcvd_keepalive +
 	    p->stats.msg_rcvd_rrefresh);
+	printf("  Update statistics:\n");
+	printf("  %-15s %-10s %-10s\n", "", "Sent", "Received");
+	printf("  %-15s %10llu %10llu\n", "Updates",
+	    p->stats.prefix_sent_update, p->stats.prefix_rcvd_update);
+	printf("  %-15s %10llu %10llu\n", "Withdraws",
+	    p->stats.prefix_sent_withdraw, p->stats.prefix_rcvd_withdraw);
 }
 
 void
@@ -868,8 +928,6 @@ ift2ifm(int media_type)
 		return (IFM_ETHER);
 	case IFT_FDDI:
 		return (IFM_FDDI);
-	case IFT_ISO88025:
-		return (IFM_TOKEN);
 	case IFT_CARP:
 		return (IFM_CARP);
 	case IFT_IEEE80211:
@@ -905,7 +963,9 @@ get_linkstate(int media_type, int link_state)
 			if (p->ifms_type != media_type ||
 			    p->ifms_valid != ifm_status_valid_list[i])
 				continue;
-			return (p->ifms_string[link_state == LINK_STATE_UP]);
+			if (LINK_STATE_IS_UP(link_state))
+				return (p->ifms_string[1]);
+			return (p->ifms_string[0]);
 		}
 
 	return ("unknown link state");
@@ -1153,7 +1213,7 @@ fmt_mem(int64_t num)
 	static char	buf[16];
 
 	if (fmt_scaled(num, buf) == -1)
-		snprintf(buf, sizeof(buf), "%lldB", num);
+		snprintf(buf, sizeof(buf), "%lldB", (long long)num);
 
 	return (buf);
 }
@@ -1168,28 +1228,29 @@ show_rib_memory_msg(struct imsg *imsg)
 		memcpy(&stats, imsg->data, sizeof(stats));
 		printf("RDE memory statistics\n");
 		printf("%10lld IPv4 network entries using %s of memory\n",
-		    stats.pt4_cnt, fmt_mem(stats.pt4_cnt *
+		    (long long)stats.pt4_cnt, fmt_mem(stats.pt4_cnt *
 		    sizeof(struct pt_entry4)));
 		if (stats.pt6_cnt != 0)
 			printf("%10lld IPv6 network entries using "
-			    "%s of memory\n", stats.pt6_cnt,
+			    "%s of memory\n", (long long)stats.pt6_cnt,
 			    fmt_mem(stats.pt6_cnt * sizeof(struct pt_entry6)));
 		printf("%10lld prefix entries using %s of memory\n",
-		    stats.prefix_cnt, fmt_mem(stats.prefix_cnt *
+		    (long long)stats.prefix_cnt, fmt_mem(stats.prefix_cnt *
 		    sizeof(struct prefix)));
 		printf("%10lld BGP path attribute entries using %s of memory\n",
-		    stats.path_cnt, fmt_mem(stats.path_cnt *
+		    (long long)stats.path_cnt, fmt_mem(stats.path_cnt *
 		    sizeof(struct rde_aspath)));
 		printf("%10lld BGP AS-PATH attribute entries using "
 		    "%s of memory,\n\t   and holding %lld references\n",
-		    stats.aspath_cnt, fmt_mem(stats.aspath_size),
-		    stats.aspath_refs);
+		    (long long)stats.aspath_cnt, fmt_mem(stats.aspath_size),
+		    (long long)stats.aspath_refs);
 		printf("%10lld BGP attributes entries using %s of memory\n",
-		    stats.attr_cnt, fmt_mem(stats.attr_cnt *
+		    (long long)stats.attr_cnt, fmt_mem(stats.attr_cnt *
 		    sizeof(struct attr)));
-		printf("\t   and holding %lld references\n", stats.attr_refs);
+		printf("\t   and holding %lld references\n",
+		    (long long)stats.attr_refs);
 		printf("%10lld BGP attributes using %s of memory\n",
-		    stats.attr_dcnt, fmt_mem(stats.attr_data));
+		    (long long)stats.attr_dcnt, fmt_mem(stats.attr_data));
 		printf("RIB using %s of memory\n", fmt_mem(
 		    stats.pt4_cnt * sizeof(struct pt_entry4) +
 		    stats.pt6_cnt * sizeof(struct pt_entry6) +
