@@ -1,4 +1,4 @@
-/*	$OpenBSD: log.c,v 1.48 2006/01/05 17:33:40 claudio Exp $ */
+/*	$OpenBSD: log.c,v 1.50 2007/04/23 13:04:24 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -33,7 +33,6 @@
 
 int	debug;
 
-char	*log_fmt_peer(const struct peer_config *);
 void	 logit(int, const char *, ...);
 
 char *
@@ -277,9 +276,14 @@ log_notification(const struct peer *peer, u_int8_t errcode, u_int8_t subcode,
 		else
 			suberrname = suberr_update_names[subcode];
 		break;
+	case ERR_CEASE:
+		if (subcode >= sizeof(suberr_cease_names)/sizeof(char *))
+			uk = 1;
+		else
+			suberrname = suberr_cease_names[subcode];
+		break;
 	case ERR_HOLDTIMEREXPIRED:
 	case ERR_FSM:
-	case ERR_CEASE:
 		uk = 1;
 		break;
 	default:
