@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.h,v 1.14 2006/08/23 08:21:11 claudio Exp $ */
+/*	$OpenBSD: parser.h,v 1.17 2007/04/06 18:36:32 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -27,6 +27,7 @@ enum actions {
 	SHOW_SUMMARY_TERSE,
 	SHOW_NEIGHBOR,
 	SHOW_NEIGHBOR_TIMERS,
+	SHOW_NEIGHBOR_TERSE,
 	SHOW_FIB,
 	SHOW_RIB,
 	SHOW_RIB_MEM,
@@ -44,7 +45,8 @@ enum actions {
 	NETWORK_ADD,
 	NETWORK_REMOVE,
 	NETWORK_FLUSH,
-	NETWORK_SHOW
+	NETWORK_SHOW,
+	IRRFILTER
 };
 
 struct parse_result {
@@ -52,6 +54,7 @@ struct parse_result {
 	struct bgpd_addr	peeraddr;
 	struct filter_as	as;
 	struct filter_set_head	set;
+	struct filter_community community;
 	char			peerdesc[PEER_DESCR_LEN];
 	int			flags;
 	enum actions		action;
