@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.c,v 1.211 2006/08/23 08:13:04 claudio Exp $ */
+/*	$OpenBSD: rde.c,v 1.227 2007/06/19 09:44:55 pyr Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -35,9 +35,10 @@
 #include "rde.h"
 #include "session.h"
 
-#define	PFD_PIPE_MAIN		0
+#define PFD_PIPE_MAIN		0
 #define PFD_PIPE_SESSION	1
-#define PFD_MRT_FILE		2
+#define PFD_PIPE_SESSION_CTL	2
+#define PFD_MRT_FILE		3
 
 void		 rde_sighdlr(int);
 void		 rde_dispatch_imsg_session(struct imsgbuf *);
@@ -57,7 +58,9 @@ void		 rde_update_err(struct rde_peer *, u_int8_t , u_int8_t,
 void		 rde_update_log(const char *,
 		     const struct rde_peer *, const struct bgpd_addr *,
 		     const struct bgpd_addr *, u_int8_t);
+void		 rde_as4byte_fixup(struct rde_peer *, struct rde_aspath *);
 int		 rde_reflector(struct rde_peer *, struct rde_aspath *);
+
 void		 rde_dump_rib_as(struct prefix *, struct rde_aspath *,pid_t,
 		     int);
 void		 rde_dump_filter(struct prefix *,
@@ -68,6 +71,12 @@ void		 rde_dump_upcall(struct pt_entry *, void *);
 void		 rde_dump_as(struct ctl_show_rib_request *);
 void		 rde_dump_prefix_upcall(struct pt_entry *, void *);
 void		 rde_dump_prefix(struct ctl_show_rib_request *);
+void		 rde_dump_community(struct ctl_show_rib_request *);
+void		 rde_dump_ctx_new(struct ctl_show_rib_request *, pid_t,
+		     enum imsg_type);
+void		 rde_dump_runner(void);
+int		 rde_dump_pending(void);
+
 void		 rde_up_dump_upcall(struct pt_entry *, void *);
 void		 rde_softreconfig_out(struct pt_entry *, void *);
 void		 rde_softreconfig_in(struct pt_entry *, void *);
@@ -78,7 +87,6 @@ void		 peer_init(u_int32_t);
 void		 peer_shutdown(void);
 void		 peer_localaddrs(struct rde_peer *, struct bgpd_addr *);
 struct rde_peer	*peer_add(u_int32_t, struct peer_config *);
-void		 peer_remove(struct rde_peer *);
 struct rde_peer	*peer_get(u_int32_t);
 void		 peer_up(u_int32_t, struct session_up *);
 void		 peer_down(u_int32_t);
@@ -102,10 +110,20 @@ struct rde_peer		 peerself;
 struct rde_peer		 peerdynamic;
 struct filter_head	*rules_l, *newrules;
 struct imsgbuf		*ibuf_se;
+struct imsgbuf		*ibuf_se_ctl;
 struct imsgbuf		*ibuf_main;
 struct mrt		*mrt;
 struct rde_memstats	 rdemem;
 
+struct rde_dump_ctx {
+	TAILQ_ENTRY(rde_dump_ctx)	entry;
+	struct pt_context		ptc;
+	struct ctl_show_rib_request	req;
+	sa_family_t			af;
+};
+
+TAILQ_HEAD(, rde_dump_ctx) rde_dump_h = TAILQ_HEAD_INITIALIZER(rde_dump_h);
+
 void
 rde_sighdlr(int sig)
 {
@@ -125,17 +143,18 @@ u_int32_t	nexthophashsize = 64;
 pid_t
 rde_main(struct bgpd_config *config, struct peer *peer_l,
     struct network_head *net_l, struct filter_head *rules,
-    struct mrt_head *mrt_l, int pipe_m2r[2], int pipe_s2r[2], int pipe_m2s[2])
+    struct mrt_head *mrt_l, int pipe_m2r[2], int pipe_s2r[2], int pipe_m2s[2],
+    int pipe_s2rctl[2], int debug)
 {
 	pid_t			 pid;
 	struct passwd		*pw;
 	struct peer		*p;
 	struct listen_addr	*la;
-	struct pollfd		 pfd[3];
+	struct pollfd		 pfd[4];
 	struct filter_rule	*f;
 	struct filter_set	*set;
 	struct nexthop		*nh;
-	int			 i;
+	int			 i, timeout;
 
 	switch (pid = fork()) {
 	case -1:
@@ -167,17 +186,21 @@ rde_main(struct bgpd_config *config, struct peer *peer_l,
 	signal(SIGTERM, rde_sighdlr);
 	signal(SIGINT, rde_sighdlr);
 	signal(SIGPIPE, SIG_IGN);
+	signal(SIGHUP, SIG_IGN);
 
 	close(pipe_s2r[0]);
+	close(pipe_s2rctl[0]);
 	close(pipe_m2r[0]);
 	close(pipe_m2s[0]);
 	close(pipe_m2s[1]);
 
 	/* initialize the RIB structures */
 	if ((ibuf_se = malloc(sizeof(struct imsgbuf))) == NULL ||
+	    (ibuf_se_ctl = malloc(sizeof(struct imsgbuf))) == NULL ||
 	    (ibuf_main = malloc(sizeof(struct imsgbuf))) == NULL)
 		fatal(NULL);
 	imsg_init(ibuf_se, pipe_s2r[1]);
+	imsg_init(ibuf_se_ctl, pipe_s2rctl[1]);
 	imsg_init(ibuf_main, pipe_m2r[1]);
 
 	/* peer list, mrt list and listener list are not used in the RDE */
@@ -220,6 +243,7 @@ rde_main(struct bgpd_config *config, struct peer *peer_l,
 	}
 
 	while (rde_quit == 0) {
+		timeout = INFTIM;
 		bzero(pfd, sizeof(pfd));
 		pfd[PFD_PIPE_MAIN].fd = ibuf_main->fd;
 		pfd[PFD_PIPE_MAIN].events = POLLIN;
@@ -231,14 +255,21 @@ rde_main(struct bgpd_config *config, struct peer *peer_l,
 		if (ibuf_se->w.queued > 0)
 			pfd[PFD_PIPE_SESSION].events |= POLLOUT;
 
-		i = 2;
+		pfd[PFD_PIPE_SESSION_CTL].fd = ibuf_se_ctl->fd;
+		pfd[PFD_PIPE_SESSION_CTL].events = POLLIN;
+		if (ibuf_se_ctl->w.queued > 0)
+			pfd[PFD_PIPE_SESSION_CTL].events |= POLLOUT;
+		else if (rde_dump_pending())
+			timeout = 0;
+
+		i = 3;
 		if (mrt && mrt->queued) {
 			pfd[PFD_MRT_FILE].fd = mrt->fd;
 			pfd[PFD_MRT_FILE].events = POLLOUT;
 			i++;
 		}
 
-		if (poll(pfd, i, INFTIM) == -1) {
+		if (poll(pfd, i, timeout) == -1) {
 			if (errno != EINTR)
 				fatal("poll error");
 			continue;
@@ -260,6 +291,14 @@ rde_main(struct bgpd_config *config, struct peer *peer_l,
 		if (pfd[PFD_PIPE_SESSION].revents & POLLIN)
 			rde_dispatch_imsg_session(ibuf_se);
 
+		if ((pfd[PFD_PIPE_SESSION_CTL].revents & POLLOUT) &&
+		    ibuf_se_ctl->w.queued)
+			if (msgbuf_write(&ibuf_se_ctl->w) < 0)
+				fatal("pipe write error");
+
+		if (pfd[PFD_PIPE_SESSION_CTL].revents & POLLIN)
+			rde_dispatch_imsg_session(ibuf_se_ctl);
+
 		if (pfd[PFD_MRT_FILE].revents & POLLOUT) {
 			if (mrt_write(mrt) == -1) {
 				free(mrt);
@@ -270,14 +309,18 @@ rde_main(struct bgpd_config *config, struct peer *peer_l,
 
 		rde_update_queue_runner();
 		rde_update6_queue_runner();
+		if (ibuf_se_ctl->w.queued <= 0)
+			rde_dump_runner();
 	}
 
-	rde_shutdown();
+	/* do not clean up on shutdown on production, it takes ages. */
+	if (debug)
+		rde_shutdown();
 
-	msgbuf_write(&ibuf_se->w);
 	msgbuf_clear(&ibuf_se->w);
 	free(ibuf_se);
-	msgbuf_write(&ibuf_main->w);
+	msgbuf_clear(&ibuf_se_ctl->w);
+	free(ibuf_se_ctl);
 	msgbuf_clear(&ibuf_main->w);
 	free(ibuf_main);
 
@@ -300,7 +343,6 @@ rde_dispatch_imsg_session(struct imsgbuf *ibuf)
 	struct ctl_show_rib_request	req;
 	struct filter_set	*s;
 	struct nexthop		*nh;
-	pid_t			 pid;
 	int			 n;
 	sa_family_t		 af = AF_UNSPEC;
 
@@ -424,11 +466,9 @@ badnet:
 				log_warnx("rde_dispatch: wrong imsg len");
 				break;
 			}
-			pid = imsg.hdr.pid;
-			memcpy(&af, imsg.data, sizeof(af));
-			pt_dump(network_dump_upcall, &pid, af);
-			imsg_compose(ibuf_se, IMSG_CTL_END, 0, pid, -1,
-			    NULL, 0);
+			bzero(&req, sizeof(req));
+			memcpy(&req.af, imsg.data, sizeof(af));
+			rde_dump_ctx_new(&req, imsg.hdr.pid, imsg.hdr.type);
 			break;
 		case IMSG_CTL_SHOW_RIB:
 			if (imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(req)) {
@@ -436,10 +476,7 @@ badnet:
 				break;
 			}
 			memcpy(&req, imsg.data, sizeof(req));
-			req.pid = imsg.hdr.pid;
-			pt_dump(rde_dump_upcall, &req, req.af);
-			imsg_compose(ibuf_se, IMSG_CTL_END, 0, req.pid, -1,
-			    NULL, 0);
+			rde_dump_ctx_new(&req, imsg.hdr.pid, imsg.hdr.type);
 			break;
 		case IMSG_CTL_SHOW_RIB_AS:
 			if (imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(req)) {
@@ -449,7 +486,7 @@ badnet:
 			memcpy(&req, imsg.data, sizeof(req));
 			req.pid = imsg.hdr.pid;
 			rde_dump_as(&req);
-			imsg_compose(ibuf_se, IMSG_CTL_END, 0, req.pid, -1,
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_END, 0, req.pid, -1,
 			    NULL, 0);
 			break;
 		case IMSG_CTL_SHOW_RIB_PREFIX:
@@ -460,7 +497,18 @@ badnet:
 			memcpy(&req, imsg.data, sizeof(req));
 			req.pid = imsg.hdr.pid;
 			rde_dump_prefix(&req);
-			imsg_compose(ibuf_se, IMSG_CTL_END, 0, req.pid, -1,
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_END, 0, req.pid, -1,
+			    NULL, 0);
+			break;
+		case IMSG_CTL_SHOW_RIB_COMMUNITY:
+			if (imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(req)) {
+				log_warnx("rde_dispatch: wrong imsg len");
+				break;
+			}
+			memcpy(&req, imsg.data, sizeof(req));
+			req.pid = imsg.hdr.pid;
+			rde_dump_community(&req);
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_END, 0, req.pid, -1,
 			    NULL, 0);
 			break;
 		case IMSG_CTL_SHOW_NEIGHBOR:
@@ -471,17 +519,26 @@ badnet:
 			}
 			memcpy(&p, imsg.data, sizeof(struct peer));
 			peer = peer_get(p.conf.id);
-			if (peer != NULL)
+			if (peer != NULL) {
 				p.stats.prefix_cnt = peer->prefix_cnt;
-			imsg_compose(ibuf_se, IMSG_CTL_SHOW_NEIGHBOR, 0,
+				p.stats.prefix_rcvd_update =
+				    peer->prefix_rcvd_update;
+				p.stats.prefix_rcvd_withdraw =
+				    peer->prefix_rcvd_withdraw;
+				p.stats.prefix_sent_update =
+				    peer->prefix_sent_update;
+				p.stats.prefix_sent_withdraw =
+				    peer->prefix_sent_withdraw;
+			}
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_SHOW_NEIGHBOR, 0,
 			    imsg.hdr.pid, -1, &p, sizeof(struct peer));
 			break;
 		case IMSG_CTL_END:
-			imsg_compose(ibuf_se, IMSG_CTL_END, 0, imsg.hdr.pid,
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_END, 0, imsg.hdr.pid,
 			    -1, NULL, 0);
 			break;
 		case IMSG_CTL_SHOW_RIB_MEM:
-			imsg_compose(ibuf_se, IMSG_CTL_SHOW_RIB_MEM, 0,
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_SHOW_RIB_MEM, 0,
 			    imsg.hdr.pid, -1, &rdemem, sizeof(rdemem));
 			break;
 		default:
@@ -644,7 +701,7 @@ rde_dispatch_imsg_parent(struct imsgbuf *ibuf)
 				    "but didn't receive any");
 
 			if (xmrt->type == MRT_TABLE_DUMP) {
-				/* do not dump if a other is still running */
+				/* do not dump if another is still running */
 				if (mrt == NULL || mrt->queued == 0) {
 					free(mrt);
 					mrt = xmrt;
@@ -674,7 +731,7 @@ rde_update_dispatch(struct imsg *imsg)
 	struct rde_peer		*peer;
 	struct rde_aspath	*asp = NULL, *fasp;
 	u_char			*p, *mpp = NULL;
-	int			 pos = 0;
+	int			 error = -1, pos = 0;
 	u_int16_t		 afi, len, mplen;
 	u_int16_t		 withdrawn_len;
 	u_int16_t		 attrpath_len;
@@ -718,10 +775,8 @@ rde_update_dispatch(struct imsg *imsg)
 		asp = path_get();
 		while (len > 0) {
 			if ((pos = rde_attr_parse(p, len, peer, asp,
-			    &mpa)) < 0) {
-				path_put(asp);
-				return (-1);
-			}
+			    &mpa)) < 0)
+				goto done;
 			p += pos;
 			len -= pos;
 		}
@@ -731,10 +786,16 @@ rde_update_dispatch(struct imsg *imsg)
 		    nlri_len))) {
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_MISSNG_WK_ATTR,
 			    &subtype, sizeof(u_int8_t));
-			path_put(asp);
-			return (-1);
+			goto done;
 		}
 
+		/*
+		 * if either ATTR_NEW_AGGREGATOR or ATTR_NEW_ASPATH is present
+		 * try to fixup the attributes.
+		 */
+		if (asp->flags & F_ATTR_AS4BYTE_NEW)
+			rde_as4byte_fixup(peer, asp);
+
 		/* enforce remote AS if requested */
 		if (asp->flags & F_ATTR_ASPATH &&
 		    peer->conf.enforce_as == ENFORCE_AS_ON)
@@ -742,13 +803,12 @@ rde_update_dispatch(struct imsg *imsg)
 			    aspath_neighbor(asp->aspath)) {
 				rde_update_err(peer, ERR_UPDATE, ERR_UPD_ASPATH,
 				    NULL, 0);
-				path_put(asp);
-				return (-1);
+				goto done;
 			}
 
 		if (rde_reflector(peer, asp) != 1) {
-			path_put(asp);
-			return (0);
+			error = 0;
+			goto done;
 		}
 	}
 
@@ -760,23 +820,19 @@ rde_update_dispatch(struct imsg *imsg)
 		if ((pos = rde_update_get_prefix(p, len, &prefix,
 		    &prefixlen)) == -1) {
 			/*
-			 * the rfc does not mention what we should do in
+			 * the RFC does not mention what we should do in
 			 * this case. Let's do the same as in the NLRI case.
 			 */
 			log_peer_warnx(&peer->conf, "bad withdraw prefix");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_NETWORK,
 			    NULL, 0);
-			if (attrpath_len != 0)
-				path_put(asp);
-			return (-1);
+			goto done;
 		}
 		if (prefixlen > 32) {
 			log_peer_warnx(&peer->conf, "bad withdraw prefix");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_NETWORK,
 			    NULL, 0);
-			if (attrpath_len != 0)
-				path_put(asp);
-			return (-1);
+			goto done;
 		}
 
 		p += pos;
@@ -787,11 +843,10 @@ rde_update_dispatch(struct imsg *imsg)
 			log_peer_warnx(&peer->conf, "bad AFI, IPv4 disabled");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_OPTATTR,
 			    NULL, 0);
-			if (attrpath_len != 0)
-				path_put(asp);
-			return (-1);
+			goto done;
 		}
 
+		peer->prefix_rcvd_withdraw++;
 		rde_update_log("withdraw", peer, NULL, &prefix, prefixlen);
 		prefix_remove(peer, &prefix, prefixlen, F_LOCAL);
 		prefix_remove(peer, &prefix, prefixlen, F_ORIGINAL);
@@ -800,7 +855,7 @@ rde_update_dispatch(struct imsg *imsg)
 	if (attrpath_len == 0) /* 0 = no NLRI information in this message */
 		return (0);
 
-	/* withdraw MP_UNREACH_NRLI if available */
+	/* withdraw MP_UNREACH_NLRI if available */
 	if (mpa.unreach_len != 0) {
 		mpp = mpa.unreach;
 		mplen = mpa.unreach_len;
@@ -817,8 +872,7 @@ rde_update_dispatch(struct imsg *imsg)
 				    "IPv6 disabled");
 				rde_update_err(peer, ERR_UPDATE,
 				    ERR_UPD_OPTATTR, NULL, 0);
-				path_put(asp);
-				return (-1);
+				goto done;
 			}
 			while (mplen > 0) {
 				if ((pos = rde_update_get_prefix6(mpp, mplen,
@@ -828,8 +882,7 @@ rde_update_dispatch(struct imsg *imsg)
 					rde_update_err(peer, ERR_UPDATE,
 					    ERR_UPD_OPTATTR,
 					    mpa.unreach, mpa.unreach_len);
-					path_put(asp);
-					return (-1);
+					goto done;
 				}
 				if (prefixlen > 128) {
 					log_peer_warnx(&peer->conf,
@@ -837,13 +890,13 @@ rde_update_dispatch(struct imsg *imsg)
 					rde_update_err(peer, ERR_UPDATE,
 					    ERR_UPD_OPTATTR,
 					    mpa.unreach, mpa.unreach_len);
-					path_put(asp);
-					return (-1);
+					goto done;
 				}
 
 				mpp += pos;
 				mplen -= pos;
 
+				peer->prefix_rcvd_withdraw++;
 				rde_update_log("withdraw", peer, NULL,
 				    &prefix, prefixlen);
 				prefix_remove(peer, &prefix, prefixlen,
@@ -858,8 +911,8 @@ rde_update_dispatch(struct imsg *imsg)
 		}
 
 		if ((asp->flags & ~F_ATTR_MP_UNREACH) == 0) {
-			path_put(asp);
-			return (0);
+			error = 0;
+			goto done;
 		}
 	}
 
@@ -868,8 +921,8 @@ rde_update_dispatch(struct imsg *imsg)
 
 	/* aspath needs to be loop free nota bene this is not a hard error */
 	if (peer->conf.ebgp && !aspath_loopfree(asp->aspath, conf->as)) {
-		path_put(asp);
-		return (0);
+		error = 0;
+		goto done;
 	}
 
 	/* parse nlri prefix */
@@ -879,15 +932,13 @@ rde_update_dispatch(struct imsg *imsg)
 			log_peer_warnx(&peer->conf, "bad nlri prefix");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_NETWORK,
 			    NULL, 0);
-			path_put(asp);
-			return (-1);
+			goto done;
 		}
 		if (prefixlen > 32) {
 			log_peer_warnx(&peer->conf, "bad nlri prefix");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_NETWORK,
 			    NULL, 0);
-			path_put(asp);
-			return (-1);
+			goto done;
 		}
 
 		p += pos;
@@ -898,10 +949,10 @@ rde_update_dispatch(struct imsg *imsg)
 			log_peer_warnx(&peer->conf, "bad AFI, IPv4 disabled");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_OPTATTR,
 			    NULL, 0);
-			path_put(asp);
-			return (-1);
+			goto done;
 		}
 
+		peer->prefix_rcvd_update++;
 		/* add original path to the Adj-RIB-In */
 		if (peer->conf.softreconfig_in)
 			path_update(peer, asp, &prefix, prefixlen, F_ORIGINAL);
@@ -919,9 +970,8 @@ rde_update_dispatch(struct imsg *imsg)
 			log_peer_warnx(&peer->conf, "prefix limit reached");
 			rde_update_err(peer, ERR_CEASE, ERR_CEASE_MAX_PREFIX,
 			    NULL, 0);
-			path_put(asp);
 			path_put(fasp);
-			return (-1);
+			goto done;
 		}
 
 		if (fasp == NULL)
@@ -949,13 +999,18 @@ rde_update_dispatch(struct imsg *imsg)
 
 		/*
 		 * this works because asp is not linked.
+		 * But first unlock the previously locked nexthop.
 		 */
+		if (asp->nexthop) {
+			asp->nexthop->refcnt--;
+			(void)nexthop_delete(asp->nexthop);
+			asp->nexthop = NULL;
+		}
 		if ((pos = rde_get_mp_nexthop(mpp, mplen, afi, asp)) == -1) {
 			log_peer_warnx(&peer->conf, "bad IPv6 nlri prefix");
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_OPTATTR,
 			    mpa.reach, mpa.reach_len);
-			path_put(asp);
-			return (-1);
+			goto done;
 		}
 		mpp += pos;
 		mplen -= pos;
@@ -967,8 +1022,7 @@ rde_update_dispatch(struct imsg *imsg)
 				    "IPv6 disabled");
 				rde_update_err(peer, ERR_UPDATE,
 				    ERR_UPD_OPTATTR, NULL, 0);
-				path_put(asp);
-				return (-1);
+				goto done;
 			}
 
 			while (mplen > 0) {
@@ -979,20 +1033,19 @@ rde_update_dispatch(struct imsg *imsg)
 					rde_update_err(peer, ERR_UPDATE,
 					    ERR_UPD_OPTATTR,
 					    mpa.reach, mpa.reach_len);
-					path_put(asp);
-					return (-1);
+					goto done;
 				}
 				if (prefixlen > 128) {
 					rde_update_err(peer, ERR_UPDATE,
 					    ERR_UPD_OPTATTR,
 					    mpa.reach, mpa.reach_len);
-					path_put(asp);
-					return (-1);
+					goto done;
 				}
 
 				mpp += pos;
 				mplen -= pos;
 
+				peer->prefix_rcvd_update++;
 				/* add original path to the Adj-RIB-In */
 				if (peer->conf.softreconfig_in)
 					path_update(peer, asp, &prefix,
@@ -1013,9 +1066,8 @@ rde_update_dispatch(struct imsg *imsg)
 					    "prefix limit reached");
 					rde_update_err(peer, ERR_CEASE,
 					    ERR_CEASE_MAX_PREFIX, NULL, 0);
-					path_put(asp);
 					path_put(fasp);
-					return (-1);
+					goto done;
 				}
 
 				if (fasp == NULL)
@@ -1038,10 +1090,18 @@ rde_update_dispatch(struct imsg *imsg)
 		}
 	}
 
-	/* need to free allocated attribute memory that is no longer used */
-	path_put(asp);
+done:
+	if (attrpath_len != 0) {
+		/* unlock the previously locked entry */
+		if (asp->nexthop) {
+			asp->nexthop->refcnt--;
+			(void)nexthop_delete(asp->nexthop);
+		}
+		/* free allocated attribute memory that is no longer used */
+		path_put(asp);
+	}
 
-	return (0);
+	return (error);
 }
 
 /*
@@ -1064,9 +1124,9 @@ rde_attr_parse(u_char *p, u_int16_t len, struct rde_peer *peer,
     struct rde_aspath *a, struct mpattr *mpa)
 {
 	struct bgpd_addr nexthop;
-	u_char		*op = p;
+	u_char		*op = p, *npath;
 	u_int32_t	 tmp32;
-	u_int16_t	 attr_len;
+	u_int16_t	 attr_len, nlen;
 	u_int16_t	 plen = 0;
 	u_int8_t	 flags;
 	u_int8_t	 type;
@@ -1125,15 +1185,22 @@ bad_flags:
 	case ATTR_ASPATH:
 		if (!CHECK_FLAGS(flags, ATTR_WELL_KNOWN, 0))
 			goto bad_flags;
-		if (aspath_verify(p, attr_len) != 0) {
+		if (aspath_verify(p, attr_len, rde_as4byte(peer)) != 0) {
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_ASPATH,
 			    NULL, 0);
 			return (-1);
 		}
 		if (a->flags & F_ATTR_ASPATH)
 			goto bad_list;
+		if (rde_as4byte(peer)) {
+			npath = p;
+			nlen = attr_len;
+		} else
+			npath = aspath_inflate(p, attr_len, &nlen);
 		a->flags |= F_ATTR_ASPATH;
-		a->aspath = aspath_get(p, attr_len);
+		a->aspath = aspath_get(npath, nlen);
+		if (npath != p)
+			free(npath);
 		plen += attr_len;
 		break;
 	case ATTR_NEXTHOP:
@@ -1159,6 +1226,12 @@ bad_flags:
 			return (-1);
 		}
 		a->nexthop = nexthop_get(&nexthop);
+		/*
+		 * lock the nexthop because it is not yet linked else
+		 * withdraws may remove this nexthop which in turn would
+		 * cause a use after free error.
+		 */
+		a->nexthop->refcnt++;
 		break;
 	case ATTR_MED:
 		if (attr_len != 4)
@@ -1196,10 +1269,23 @@ bad_flags:
 			goto bad_flags;
 		goto optattr;
 	case ATTR_AGGREGATOR:
-		if (attr_len != 6)
+		if ((!rde_as4byte(peer) && attr_len != 6) ||
+		    (rde_as4byte(peer) && attr_len != 8))
 			goto bad_len;
 		if (!CHECK_FLAGS(flags, ATTR_OPTIONAL|ATTR_TRANSITIVE, 0))
 			goto bad_flags;
+		if (!rde_as4byte(peer)) {
+			/* need to inflate aggregator AS to 4-byte */
+			u_char	t[8];
+			t[0] = t[1] = 0;
+			UPD_READ(&t[2], p, plen, 2);
+			UPD_READ(&t[4], p, plen, 4);
+			if (attr_optadd(a, flags, type, t,
+			    sizeof(t)) == -1)
+				goto bad_list;
+			break;
+		}
+		/* 4-byte ready server take the default route */
 		goto optattr;
 	case ATTR_COMMUNITIES:
 		if ((attr_len & 0x3) != 0)
@@ -1248,6 +1334,26 @@ bad_flags:
 		mpa->unreach_len = attr_len;
 		plen += attr_len;
 		break;
+	case ATTR_NEW_AGGREGATOR:
+		if (attr_len != 8)
+			goto bad_len;
+		if (!CHECK_FLAGS(flags, ATTR_OPTIONAL|ATTR_TRANSITIVE,
+		    ATTR_PARTIAL))
+			goto bad_flags;
+		a->flags |= F_ATTR_AS4BYTE_NEW;
+		goto optattr;
+	case ATTR_NEW_ASPATH:
+		if (!CHECK_FLAGS(flags, ATTR_OPTIONAL|ATTR_TRANSITIVE,
+		    ATTR_PARTIAL))
+			goto bad_flags;
+		if (aspath_verify(p, attr_len, 1) != 0) {
+			/* XXX draft does not specify how to handle errors */
+			rde_update_err(peer, ERR_UPDATE, ERR_UPD_ASPATH,
+			    NULL, 0);
+			return (-1);
+		}
+		a->flags |= F_ATTR_AS4BYTE_NEW;
+		goto optattr;
 	default:
 		if ((flags & ATTR_OPTIONAL) == 0) {
 			rde_update_err(peer, ERR_UPDATE, ERR_UPD_UNKNWN_WK_ATTR,
@@ -1327,15 +1433,18 @@ rde_get_mp_nexthop(u_char *data, u_int16_t len, u_int16_t afi,
 		nexthop.af = AF_INET6;
 		memcpy(&nexthop.v6.s6_addr, data, 16);
 		asp->nexthop = nexthop_get(&nexthop);
+		/*
+		 * lock the nexthop because it is not yet linked else
+		 * withdraws may remove this nexthop which in turn would
+		 * cause a use after free error.
+		 */
+		asp->nexthop->refcnt++;
 
-		totlen += nhlen;
-		data += nhlen;
+		/* ignore reserved (old SNPA) field as per RFC 4760 */
+		totlen += nhlen + 1;
+		data += nhlen + 1;
 
-		if (*data != 0) {
-			log_warnx("SNPA are not supported for IPv6");
-			return (-1);
-		}
-		return (++totlen);
+		return (totlen);
 	default:
 		log_warnx("bad multiprotocol nexthop, bad AF");
 		break;
@@ -1433,31 +1542,82 @@ rde_update_log(const char *message,
     const struct rde_peer *peer, const struct bgpd_addr *next,
     const struct bgpd_addr *prefix, u_int8_t prefixlen)
 {
-	char		*nexthop = NULL;
+	char		*n = NULL;
 	char		*p = NULL;
 
 	if (!(conf->log & BGPD_LOG_UPDATES))
 		return;
 
 	if (next != NULL)
-		if (asprintf(&nexthop, " via %s",
-		    log_addr(next)) == -1)
-			nexthop = NULL;
-
+		if (asprintf(&n, " via %s", log_addr(next)) == -1)
+			n = NULL;
 	if (asprintf(&p, "%s/%u", log_addr(prefix), prefixlen) == -1)
 		p = NULL;
-	log_info("neighbor %s (AS%u) %s %s %s",
-	    log_addr(&peer->conf.remote_addr), peer->conf.remote_as, message,
-	    p ? p : "out of memory", nexthop ? nexthop : "");
+	log_info("%s AS%s: %s %s%s",
+	    log_fmt_peer(&peer->conf), log_as(peer->conf.remote_as), message,
+	    p ? p : "out of memory", n ? n : "");
 
-	free(nexthop);
+	free(n);
 	free(p);
 }
 
+/*
+ * 4-Byte ASN helper function.
+ * Two scenarios need to be considered:
+ * - NEW session with NEW attributes present -> just remove the attributes
+ * - OLD session with NEW attributes present -> try to merge them
+ */
+void
+rde_as4byte_fixup(struct rde_peer *peer, struct rde_aspath *a)
+{
+	struct attr	*nasp, *naggr, *oaggr;
+	u_int32_t	 as;
+
+	/* first get the attributes */
+	nasp = attr_optget(a, ATTR_NEW_ASPATH);
+	naggr = attr_optget(a, ATTR_NEW_AGGREGATOR);
+
+	if (rde_as4byte(peer)) {
+		/* NEW session using 4-byte ASNs */
+		if (nasp)
+			attr_free(a, nasp);
+		if (naggr)
+			attr_free(a, naggr);
+		return;
+	}
+	/* OLD session using 2-byte ASNs */
+	/* try to merge the new attributes into the old ones */
+	if ((oaggr = attr_optget(a, ATTR_AGGREGATOR))) {
+		memcpy(&as, oaggr->data, sizeof(as));
+		if (ntohl(as) != AS_TRANS) {
+			/* per RFC draft ignore NEW_ASPATH and NEW_AGGREGATOR */
+			if (nasp)
+				attr_free(a, nasp);
+			if (naggr)
+				attr_free(a, naggr);
+			return;
+		}
+		if (naggr) {
+			/* switch over to new AGGREGATOR */
+			attr_free(a, oaggr);
+			if (attr_optadd(a, ATTR_OPTIONAL | ATTR_TRANSITIVE,
+			    ATTR_AGGREGATOR, naggr->data, naggr->len))
+				fatalx("attr_optadd failed but impossible");
+		}
+	}
+	/* there is no need for NEW_AGGREGATOR any more */
+	if (naggr)
+		attr_free(a, naggr);
+
+	/* merge NEW_ASPATH with ASPATH */
+	if (nasp)
+		aspath_merge(a, nasp);
+}
+
+
 /*
  * route reflector helper function
  */
-
 int
 rde_reflector(struct rde_peer *peer, struct rde_aspath *asp)
 {
@@ -1560,22 +1720,23 @@ rde_dump_rib_as(struct prefix *p, struct rde_aspath *asp, pid_t pid, int flags)
 		rib.flags |= F_RIB_ELIGIBLE;
 	rib.aspath_len = aspath_length(asp->aspath);
 
-	if ((wbuf = imsg_create(ibuf_se, IMSG_CTL_SHOW_RIB, 0, pid,
+	if ((wbuf = imsg_create(ibuf_se_ctl, IMSG_CTL_SHOW_RIB, 0, pid,
 	    sizeof(rib) + rib.aspath_len)) == NULL)
 		return;
 	if (imsg_add(wbuf, &rib, sizeof(rib)) == -1 ||
 	    imsg_add(wbuf, aspath_dump(asp->aspath),
 	    rib.aspath_len) == -1)
 		return;
-	if (imsg_close(ibuf_se, wbuf) == -1)
+	if (imsg_close(ibuf_se_ctl, wbuf) == -1)
 		return;
 
 	if (flags & F_CTL_DETAIL)
 		for (l = 0; l < asp->others_len; l++) {
 			if ((a = asp->others[l]) == NULL)
 				break;
-			if ((wbuf = imsg_create(ibuf_se, IMSG_CTL_SHOW_RIB_ATTR,
-			    0, pid, attr_optlen(a))) == NULL)
+			if ((wbuf = imsg_create(ibuf_se_ctl,
+			    IMSG_CTL_SHOW_RIB_ATTR, 0, pid,
+			    attr_optlen(a))) == NULL)
 				return;
 			if ((bp = buf_reserve(wbuf, attr_optlen(a))) == NULL) {
 				buf_free(wbuf);
@@ -1586,7 +1747,7 @@ rde_dump_rib_as(struct prefix *p, struct rde_aspath *asp, pid_t pid, int flags)
 				buf_free(wbuf);
 				return;
 			}
-			if (imsg_close(ibuf_se, wbuf) == -1)
+			if (imsg_close(ibuf_se_ctl, wbuf) == -1)
 				return;
 		}
 }
@@ -1634,7 +1795,7 @@ rde_dump_filter(struct prefix *p, struct ctl_show_rib_request *req)
 			return;
 
 		if (req->peerid) {
-		       	if ((peer = peer_get(req->peerid)) != NULL)
+			if ((peer = peer_get(req->peerid)) != NULL)
 				rde_dump_filterout(peer, p, req);
 			return;
 		}
@@ -1706,6 +1867,88 @@ rde_dump_prefix(struct ctl_show_rib_request *req)
 	}
 }
 
+void
+rde_dump_community(struct ctl_show_rib_request *req)
+{
+	extern struct path_table	 pathtable;
+	struct rde_aspath		*asp;
+	struct prefix			*p;
+	u_int32_t			 i;
+
+	for (i = 0; i <= pathtable.path_hashmask; i++) {
+		LIST_FOREACH(asp, &pathtable.path_hashtbl[i], path_l) {
+			if (!rde_filter_community(asp, req->community.as,
+			    req->community.type))
+				continue;
+			/* match found */
+			LIST_FOREACH(p, &asp->prefix_h, path_l)
+				rde_dump_filter(p, req);
+		}
+	}
+}
+
+void
+rde_dump_ctx_new(struct ctl_show_rib_request *req, pid_t pid,
+    enum imsg_type type)
+{
+	struct rde_dump_ctx	*ctx;
+	u_int			 error;
+
+	if ((ctx = calloc(1, sizeof(*ctx))) == NULL) {
+		log_warn("rde_dump_ctx_new");
+		error = CTL_RES_NOMEM;
+		imsg_compose(ibuf_se_ctl, IMSG_CTL_RESULT, 0, pid, -1, &error,
+		    sizeof(error));
+		return;
+	}
+	memcpy(&ctx->req, req, sizeof(struct ctl_show_rib_request));
+	ctx->req.pid = pid;
+	ctx->req.type = type;
+	ctx->ptc.count = RDE_RUNNER_ROUNDS;
+	ctx->af = ctx->req.af;
+	if (ctx->af == AF_UNSPEC)
+		ctx->af = AF_INET;
+
+	TAILQ_INSERT_TAIL(&rde_dump_h, ctx, entry);
+}
+
+void
+rde_dump_runner(void)
+{
+	struct rde_dump_ctx	*ctx, *next;
+
+	for (ctx = TAILQ_FIRST(&rde_dump_h); ctx != NULL; ctx = next) {
+		next = TAILQ_NEXT(ctx, entry);
+		if (ctx->ptc.done) {
+			imsg_compose(ibuf_se_ctl, IMSG_CTL_END, 0, ctx->req.pid,
+			    -1, NULL, 0);
+			TAILQ_REMOVE(&rde_dump_h, ctx, entry);
+			free(ctx);
+			continue;
+		}
+		switch (ctx->req.type) {
+		case IMSG_CTL_SHOW_NETWORK:
+			pt_dump_r(network_dump_upcall, &ctx->req.pid,
+			    ctx->af, &ctx->ptc);
+			break;
+		case IMSG_CTL_SHOW_RIB:
+			pt_dump_r(rde_dump_upcall, &ctx->req, ctx->af,
+			    &ctx->ptc);
+			break;
+		default:
+			fatalx("rde_dump_runner: unsupported imsg type");
+		}
+		if (ctx->ptc.done && ctx->req.af == AF_UNSPEC)
+			ctx->af = AF_INET6;
+	}
+}
+
+int
+rde_dump_pending(void)
+{
+	return (!TAILQ_EMPTY(&rde_dump_h));
+}
+
 /*
  * kroute specific functions
  */
@@ -1904,7 +2147,7 @@ rde_softreconfig_in(struct pt_entry *pt, void *ptr)
 	struct bgpd_addr	 addr;
 
 	pt_getaddr(pt, &addr);
-	for(p = LIST_FIRST(&pt->prefix_h); p != NULL; p = np) {
+	for (p = LIST_FIRST(&pt->prefix_h); p != NULL; p = np) {
 		np = LIST_NEXT(p, prefix_l);
 		if (!(p->flags & F_ORIGINAL))
 			continue;
@@ -1991,7 +2234,7 @@ void
 rde_update_queue_runner(void)
 {
 	struct rde_peer		*peer;
-	int			 r, sent;
+	int			 r, sent, max = RDE_RUNNER_ROUNDS;
 	u_int16_t		 len, wd_len, wpos;
 
 	len = sizeof(queue_buf) - MSGSIZE_HEADER;
@@ -2028,15 +2271,16 @@ rde_update_queue_runner(void)
 				fatal("imsg_compose error");
 			sent++;
 		}
-	} while (sent != 0);
+		max -= sent;
+	} while (sent != 0 && max > 0);
 }
 
 void
 rde_update6_queue_runner(void)
 {
 	struct rde_peer		*peer;
-	char			*b;
-	int			 sent;
+	u_char			*b;
+	int			 sent, max = RDE_RUNNER_ROUNDS / 2;
 	u_int16_t		 len;
 
 	/* first withdraws ... */
@@ -2056,9 +2300,11 @@ rde_update6_queue_runner(void)
 				fatal("imsg_compose error");
 			sent++;
 		}
-	} while (sent != 0);
+		max -= sent;
+	} while (sent != 0 && max > 0);
 
 	/* ... then updates */
+	max = RDE_RUNNER_ROUNDS / 2;
 	do {
 		sent = 0;
 		LIST_FOREACH(peer, &peerlist, peer_l) {
@@ -2075,13 +2321,14 @@ rde_update6_queue_runner(void)
 				fatal("imsg_compose error");
 			sent++;
 		}
-	} while (sent != 0);
+		max -= sent;
+	} while (sent != 0 && max > 0);
 }
 
 /*
  * generic helper function
  */
-u_int16_t
+u_int32_t
 rde_local_as(void)
 {
 	return (conf->as);
@@ -2103,6 +2350,12 @@ rde_decisionflags(void)
 	return (conf->flags & BGPD_FLAG_DECISION_MASK);
 }
 
+int
+rde_as4byte(struct rde_peer *peer)
+{
+	return (peer->capa_announced.as4byte && peer->capa_received.as4byte);
+}
+
 /*
  * peer functions
  */
@@ -2186,14 +2439,6 @@ peer_add(u_int32_t id, struct peer_config *p_conf)
 	return (peer);
 }
 
-void
-peer_remove(struct rde_peer *peer)
-{
-	LIST_REMOVE(peer, hash_l);
-	LIST_REMOVE(peer, peer_l);
-	free(peer);
-}
-
 void
 peer_localaddrs(struct rde_peer *peer, struct bgpd_addr *laddr)
 {
@@ -2268,6 +2513,7 @@ peer_up(u_int32_t id, struct session_up *sup)
 	if (peer->state != PEER_DOWN && peer->state != PEER_NONE)
 		fatalx("peer_up: bad state");
 	peer->remote_bgpid = ntohl(sup->remote_bgpid);
+	peer->short_as = sup->short_as;
 	memcpy(&peer->remote_addr, &sup->remote_addr,
 	    sizeof(peer->remote_addr));
 	memcpy(&peer->capa_announced, &sup->capa_announced,
@@ -2315,7 +2561,9 @@ peer_down(u_int32_t id)
 	/* Deletions are performed in path_remove() */
 	rde_send_pftable_commit();
 
-	peer_remove(peer);
+	LIST_REMOVE(peer, hash_l);
+	LIST_REMOVE(peer, peer_l);
+	free(peer);
 }
 
 void
@@ -2330,8 +2578,7 @@ peer_dump(u_int32_t id, u_int16_t afi, u_int8_t safi)
 	}
 
 	if (afi == AFI_ALL || afi == AFI_IPv4)
-		if (safi == SAFI_ALL || safi == SAFI_UNICAST ||
-		    safi == SAFI_BOTH) {
+		if (safi == SAFI_ALL || safi == SAFI_UNICAST) {
 			if (peer->conf.announce_type ==
 			    ANNOUNCE_DEFAULT_ROUTE)
 				up_generate_default(rules_l, peer, AF_INET);
@@ -2339,8 +2586,7 @@ peer_dump(u_int32_t id, u_int16_t afi, u_int8_t safi)
 				pt_dump(rde_up_dump_upcall, peer, AF_INET);
 		}
 	if (afi == AFI_ALL || afi == AFI_IPv6)
-		if (safi == SAFI_ALL || safi == SAFI_UNICAST ||
-		    safi == SAFI_BOTH) {
+		if (safi == SAFI_ALL || safi == SAFI_UNICAST) {
 			if (peer->conf.announce_type ==
 			    ANNOUNCE_DEFAULT_ROUTE)
 				up_generate_default(rules_l, peer, AF_INET6);
@@ -2348,10 +2594,8 @@ peer_dump(u_int32_t id, u_int16_t afi, u_int8_t safi)
 				pt_dump(rde_up_dump_upcall, peer, AF_INET6);
 		}
 
-#if 0
-	/* do not send EOR for now, juniper apparently has a problem with it */
-	peer_send_eor(peer, afi, safi);
-#endif
+	if (peer->capa_received.restart && peer->capa_announced.restart)
+		peer_send_eor(peer, afi, safi);
 }
 
 /* End-of-RIB marker, draft-ietf-idr-restart-13.txt */
@@ -2401,12 +2645,14 @@ network_init(struct network_head *net_l)
 	peerself.remote_bgpid = ntohl(conf->bgpid);
 	id.s_addr = conf->bgpid;
 	peerself.conf.remote_as = conf->as;
+	peerself.short_as = conf->short_as;
 	snprintf(peerself.conf.descr, sizeof(peerself.conf.descr),
 	    "LOCAL: ID %s", inet_ntoa(id));
 	bzero(&peerdynamic, sizeof(peerdynamic));
 	peerdynamic.state = PEER_UP;
 	peerdynamic.remote_bgpid = ntohl(conf->bgpid);
 	peerdynamic.conf.remote_as = conf->as;
+	peerdynamic.short_as = conf->short_as;
 	snprintf(peerdynamic.conf.descr, sizeof(peerdynamic.conf.descr),
 	    "LOCAL: ID %s", inet_ntoa(id));
 
@@ -2478,8 +2724,8 @@ network_dump_upcall(struct pt_entry *pt, void *ptr)
 			k.prefixlen = p->prefix->prefixlen;
 			if (p->aspath->peer == &peerself)
 				k.flags = F_KERNEL;
-			if (imsg_compose(ibuf_se, IMSG_CTL_SHOW_NETWORK, 0, pid,
-			    -1, &k, sizeof(k)) == -1)
+			if (imsg_compose(ibuf_se_ctl, IMSG_CTL_SHOW_NETWORK, 0,
+			    pid, -1, &k, sizeof(k)) == -1)
 				log_warnx("network_dump_upcall: "
 				    "imsg_compose error");
 		}
@@ -2490,7 +2736,7 @@ network_dump_upcall(struct pt_entry *pt, void *ptr)
 			k6.prefixlen = p->prefix->prefixlen;
 			if (p->aspath->peer == &peerself)
 				k6.flags = F_KERNEL;
-			if (imsg_compose(ibuf_se, IMSG_CTL_SHOW_NETWORK6, 0,
+			if (imsg_compose(ibuf_se_ctl, IMSG_CTL_SHOW_NETWORK6, 0,
 			    pid, -1, &k6, sizeof(k6)) == -1)
 				log_warnx("network_dump_upcall: "
 				    "imsg_compose error");
@@ -2577,7 +2823,7 @@ sa_cmp(struct bgpd_addr *a, struct sockaddr *b)
 		break;
 	case AF_INET6:
 		in6_b = (struct sockaddr_in6 *)b;
-#if defined(__KAME__) && defined(KAME_SCOPEID)
+#ifdef __KAME__
 		/* directly stolen from sbin/ifconfig/ifconfig.c */
 		if (IN6_IS_ADDR_LINKLOCAL(&in6_b->sin6_addr)) {
 			in6_b->sin6_scope_id =
