@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.h,v 1.86 2006/08/27 16:11:05 henning Exp $ */
+/*	$OpenBSD: session.h,v 1.92 2007/04/23 13:04:24 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -104,7 +104,8 @@ enum capa_codes {
 	CAPA_NONE,
 	CAPA_MP,
 	CAPA_REFRESH,
-	CAPA_RESTART = 64
+	CAPA_RESTART = 64,
+	CAPA_AS4BYTE = 65
 };
 
 struct bgp_msg {
@@ -152,6 +153,10 @@ struct peer_stats {
 	u_int64_t		 msg_sent_notification;
 	u_int64_t		 msg_sent_keepalive;
 	u_int64_t		 msg_sent_rrefresh;
+	u_int64_t		 prefix_rcvd_update;
+	u_int64_t		 prefix_rcvd_withdraw;
+	u_int64_t		 prefix_sent_update;
+	u_int64_t		 prefix_sent_withdraw;
 	time_t			 last_updown;
 	time_t			 last_read;
 	u_int32_t		 prefix_cnt;
@@ -166,6 +171,13 @@ struct peer {
 		struct capabilities	ann;
 		struct capabilities	peer;
 	}			 capa;
+	struct {
+		struct bgpd_addr	local_addr;
+		u_int32_t		spi_in;
+		u_int32_t		spi_out;
+		enum auth_method	method;
+		u_int8_t		established;
+	} auth;
 	struct sockaddr_storage	 sa_local;
 	struct sockaddr_storage	 sa_remote;
 	struct msgbuf		 wbuf;
@@ -183,8 +195,8 @@ struct peer {
 	u_int32_t		 remote_bgpid;
 	enum session_state	 state;
 	enum session_state	 prev_state;
+	u_int16_t		 short_as;
 	u_int16_t		 holdtime;
-	u_int8_t		 auth_established;
 	u_int8_t		 depend_ok;
 	u_int8_t		 demoted;
 	u_int8_t		 passive;
@@ -196,7 +208,7 @@ struct peer	*peers;
 void		 session_socket_blockmode(int, enum blockmodes);
 pid_t		 session_main(struct bgpd_config *, struct peer *,
 		    struct network_head *, struct filter_head *,
-		    struct mrt_head *, int[2], int[2], int[2]);
+		    struct mrt_head *, int[2], int[2], int[2], int[2]);
 void		 bgp_fsm(struct peer *, enum session_events);
 int		 session_neighbor_rrefresh(struct peer *p);
 struct peer	*getpeerbyaddr(struct bgpd_addr *);
@@ -205,6 +217,7 @@ int		 imsg_compose_parent(int, pid_t, void *, u_int16_t);
 int		 imsg_compose_rde(int, pid_t, void *, u_int16_t);
 
 /* log.c */
+char		*log_fmt_peer(const struct peer_config *);
 void		 log_statechange(struct peer *, enum session_state,
 		    enum session_events);
 void		 log_notification(const struct peer *, u_int8_t, u_int8_t,
@@ -222,7 +235,8 @@ void	 prepare_listeners(struct bgpd_config *);
 
 /* rde.c */
 pid_t	 rde_main(struct bgpd_config *, struct peer *, struct network_head *,
-	    struct filter_head *, struct mrt_head *, int[2], int[2], int[2]);
+	    struct filter_head *, struct mrt_head *, int[2], int[2], int[2],
+	    int[2], int);
 
 /* control.c */
 int	control_init(int, char *);
